/*
 *  VuzeMacDockPluginjnilib.c
 *  VuzeMacDockPlugin
 *
 * Copyright (c) 2009 Marc Haisenko <marc@darkdust.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * Neither the name of the Marc Haisenko nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSE-
 * QUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include "net_darkdust_VuzeMacDockPlugin_VuzeMacDockPlugin.h"
#include <ApplicationServices/ApplicationServices.h>
#include <Cocoa/Cocoa.h>
#include <Carbon/Carbon.h>

// Overlay image.
CGImageRef overlayImage = NULL;
// Copy of the raw PNG data. Unfortunately the image is not immediately read and we thus need to
// hold a copy of the image. As it's only a few kilobyte it's not a big deal.
void *overlayImageData = NULL;


// Initialize the overlay image.
JNIEXPORT jint JNICALL  Java_net_darkdust_VuzeMacDockPlugin_VuzeMacDockPlugin_initializeDockIcon(JNIEnv *env, jobject this, jbyteArray data, jlong size)
{
	jbyte *dataRaw;
	
	// Create a copy of the raw data (see comment for overlayImageData above).
	dataRaw = (*env)->GetByteArrayElements(env, data, NULL);	
	overlayImageData = malloc(size);
	memcpy(overlayImageData, dataRaw, size);
	(*env)->ReleaseByteArrayElements(env, data, dataRaw, 0);
	
	// Encapsulate the data for Quartz.
	CGDataProviderRef source = CGDataProviderCreateWithData(NULL, overlayImageData, size, NULL);
	if (!source)
	{
		fprintf(stderr, "VuzeMacDockPlugin: Creating the data provider failed !\n");
		goto error;
	}
	
	overlayImage = CGImageCreateWithPNGDataProvider(source, NULL, true, kCGRenderingIntentDefault);
	CGDataProviderRelease(source);
	
	if (!overlayImage)
	{
		fprintf(stderr, "VuzeMacDockPlugin: Creating the image failed !\n");
		goto error;
	}
	
	// overlayImage never gets released. Mark it as being MINE ! Muahahaha !
	CGImageRetain(overlayImage);
	
	// All right, everything is fine.
	return 1;
	
error:
	// Awww, damnit, had a problem.
	if (overlayImageData)
	{
		free(overlayImageData);
		overlayImageData = NULL;
	}
	return 0;	
}

// The most important method: update the dock icon !
JNIEXPORT jint JNICALL  Java_net_darkdust_VuzeMacDockPlugin_VuzeMacDockPlugin_updateDockIcon(JNIEnv *env, jobject this, jlong up, jlong down)
{
	CGContextRef context;
	CGColorSpaceRef colorspace;
	CGImageRef image;
	OSStatus ret;
	CGRect rect;
	char strBuf[256];
    static NSImage *applicationIcon = nil;
    NSImage *icon;
	
	colorspace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);

	// Create the Quartz context into which we will draw.
	context = CGBitmapContextCreate(NULL, 500, 500, 8, (500 * 8 * 4), colorspace, kCGBitmapByteOrder32Host | kCGImageAlphaPremultipliedLast);
	
	// Copy the image into the context.
	rect.origin.x = 0;
	rect.origin.y = 0;
	rect.size.width = 500;
	rect.size.height = 500;
	CGContextDrawImage(context, rect, overlayImage);
	
	// Select a font. I've read that this can take a "considerable" amount of time but I haven't
	// found a way to cache it or something.
	CGContextSetRGBFillColor(context, 1.0, 1.0, 1.0, 1.0);
	CGContextSelectFont(context, "Monaco", 80, kCGEncodingMacRoman);

	// Draw the rates.
	sprintf(strBuf, "%7ld", (long) down / 1024);
	CGContextShowTextAtPoint(context, 90, 40, strBuf, strlen(strBuf));
	sprintf(strBuf, "%7ld", (long) up / 1024);
	CGContextShowTextAtPoint(context, 90, 160, strBuf, strlen(strBuf));
	
	// Convert the context to an image and draw the dock tile overlay.
	image = CGBitmapContextCreateImage(context);
    
    // Update the application icon.
    if (!applicationIcon) {
        // On first run, get the original icon.
        applicationIcon = [[[NSApplication sharedApplication] applicationIconImage] retain];
    }
    icon = [applicationIcon copy];
    [icon lockFocus];
    CGContextRef ctxt = [[NSGraphicsContext currentContext] graphicsPort];
    CGContextDrawImage(ctxt, CGRectMake(0, 0, [icon size].width, [icon size].height), image);
    [icon unlockFocus];
    [[NSApplication sharedApplication] setApplicationIconImage:icon];
    [icon release];
	
	// Be nice, clean up.
	CGImageRelease(image);
	CGContextRelease(context);
	CGColorSpaceRelease(colorspace);
	
	return noErr;
}
